"""Entry point.

"""
import pickle
from os.path import exists
from typing import List, Tuple

from cml.usecases.query import KnowledgeSearchUsecase
from cml.usecases.command import (
    PreprocessUsecase,
    FeatureSelectionUsecase,
    ReconstructionUsecase,
    ConstructionUsecase,
    DeconstructionUsecase)
from cml.ports.source_adapters import PandasAdapter, Adapter
from cml.ports.ml_adapter import (
    ConstructionClusteringMLModel,
    FilterMethod,
    EmbeddedMethod)
from cml.shared.settings import Settings
from cml.shared.settings import specific_settings_factory, read_settings
from cml.shared.request import (
    PreprocessRequest,
    KnowledgeSearchRequest,
    ConstructionRequest,
    FeatureSelectionRequest,
    ReconstructionRequest,
    DeconstructionRequest)
from cml.ports.ml_adapter import (
    KernelDensityEstimator,
    find_relative_extrema,
    Autoencoder,
    ReconstructionConceptualMLModel,
    ReconstructionProceduralMLModel)
from cml.domain import *


__all__ = (
    "load_settings",
    "get_settings",
    "get_source",
    "construction",
    "reconstruction",
    "reconstruction",
    "deconstruction",
    "search_knowledge",
    "visualize",
    "load_knowledge",
    "feature_selection"
)


def load_settings(path: str):
    assert isinstance(path, str)
    assert exists(path)
    read_settings(path)


def get_settings():
    return Settings


def load_knowledge(path: str):
    assert isinstance(path, str)
    assert exists(path)
    with open(path, "rb") as file:
        return pickle.load(file)


def get_source(path: str = None):
    general_settings = specific_settings_factory("general")
    preprocessing_settings = specific_settings_factory("preprocessing")
    block_processing_settings = specific_settings_factory("block_processing")
    if path: general_settings.input_file = path
    density_estimator_interface = KernelDensityEstimator
    relative_extrema_interface = find_relative_extrema
    request = PreprocessRequest(PandasAdapter,
                                general_settings,
                                preprocessing_settings,
                                block_processing_settings,
                                density_estimator_interface,
                                relative_extrema_interface)
    usecase = PreprocessUsecase()
    return usecase.execute(request)


def construction(mode: str, algorithms: List[Tuple]):
    assert mode == "conceptual"
    interface = ConstructionClusteringMLModel
    settings = specific_settings_factory("construction")
    request = ConstructionRequest(settings, algorithms, interface, mode)
    usecase = ConstructionUsecase()
    return usecase.execute(request)


def feature_selection(*, filter_method, embedded_method):
    assert filter_method is not None
    assert embedded_method is not None
    filter_method = FilterMethod(filter_method)
    embedded_method = EmbeddedMethod(embedded_method)
    settings = specific_settings_factory("feature_selection")
    request = FeatureSelectionRequest(filter_method, embedded_method, settings)
    usecase = FeatureSelectionUsecase()
    return usecase.execute(request)


def reconstruction(mode, algorithms: List[Tuple]):
    assert mode == "conceptual"
    interface = ReconstructionConceptualMLModel
    settings = specific_settings_factory("reconstruction")
    request = ReconstructionRequest(settings, algorithms, interface, mode)
    usecase = ReconstructionUsecase()
    return usecase.execute(request)


def deconstruction(mode, source, knowledge=None):
    assert mode == "conceptual"
    settings = specific_settings_factory("deconstruction")
    request = DeconstructionRequest(mode, source, settings, knowledge)
    usecase = DeconstructionUsecase()
    return usecase.execute(request)


def search_knowledge(constructor,
                     selector,
                     reconstructor,
                     deconstructor,
                     *,
                     stdout=False,
                     parallel=False):
    assert isinstance(constructor, Constructor)
    assert isinstance(selector, FeatureSelector)
    assert isinstance(reconstructor, Reconstructor)
    assert isinstance(deconstructor, Deconstructor)
    assert isinstance(stdout, bool)
    assert isinstance(parallel, bool)
    request = KnowledgeSearchRequest(constructor, selector, reconstructor,
                                     deconstructor, stdout, parallel)
    usecase = KnowledgeSearchUsecase()
    return usecase.execute(request)


def visualize():
    pass
